import json
import os
from enum import Enum, unique
from typing import List, Tuple, Union

Option_type = Tuple[object, str]
Options_type = List[Option_type]


@unique
class Coloring(Enum):
    grayscale = "Grayscale"
    custom = "custom"
    custom_random = "custom_random"
    spectrum = "Spectrum"
    overlay_positives = "Overlay (Positives)"
    overlay_negatives = "Overlay (Negatives)"
    hot_n_cold = "Hot-and-Cold"
    gold = "Gold"
    red = "Red Overlay"
    green = "Green Overlay"
    blue = "Blue Overlay"


@unique
class Region(Enum):
    right = "right"
    left = "left"
    center = "center"
    top = "top"
    bottom = "bottom"
    none = ""


@unique
class ToolWidget(Enum):
    # There are more, but I suspect they are no longer supported
    papaya = "papaya"
    html_element = "html_element"
    html_inject = "html_inject"
    action_block = "action_block"


@unique
class LayoutWidget(Enum):
    split = "split"
    tab = "tab"
    tool = "tool"


@unique
class OrientationLayout(Enum):
    vertical = "vertical"
    horizontal = "horizontal"
    none = ""


class ToolScreen:
    def __init__(self, tool_code: ToolWidget, title: str = "", config=None):
        if config is None:
            config = {}
        self.type = "tool"
        self.tool_code = tool_code.value
        self.title = title
        self.config = config


class LayoutScreen:
    def __init__(
        self,
        element_type: LayoutWidget,
        tool: ToolWidget,
        region: Region,
        load_element: int = 0,
        children=None,
        width: str = "",
        height: str = "",
        button_label: str = "",
        orientation: OrientationLayout = OrientationLayout.none,
    ):
        if children is None:
            children = list()
        self.type = element_type.value
        self.tool = tool.value
        self.region = region.value
        self.orientation = orientation.value
        self.children = children
        self.width = width
        self.height = height
        self.button_label = button_label
        self.load_element = load_element


class PapayaViewer:
    def __init__(
        self,
        title,
        width: str = "",
        height: str = "",
        button_label: str = "",
        region: Region = Region.none,
    ):
        self.title = title
        self.region = region
        self.config = {"images": list(), "title": title}
        self.load_element = 0
        self.height = height
        self.width = width
        self.button_label = button_label

    def add_file(self, file: str, coloring: Coloring):
        self.config["images"].append(
            {"file": file, "coloring": coloring.value}
        )

    def get_tool(self):
        return ToolScreen(tool_code=ToolWidget.papaya, config=self.config)

    def get_screen(self):
        return LayoutScreen(
            element_type=LayoutWidget.tool,
            tool=ToolWidget.papaya,
            load_element=self.load_element,
            region=self.region,
            height=self.height,
            width=self.width,
            button_label=self.button_label,
        )

    def __str__(self):
        return f"Papaya load element {self.load_element}"


class HtmlInject:
    def __init__(
        self,
        width: str = "",
        height: str = "",
        button_label: str = "",
        region: Region = Region.none,
    ):
        self.config = {"file": ""}
        self.load_element = 0
        self.height = height
        self.width = width
        self.button_label = button_label
        self.region = region
        self.type = ToolWidget.html_inject
        self.output_files = list()

    def add_html(self, file: str):
        self.config["file"] = file

    def get_tool(self):
        return ToolScreen(tool_code=self.type, config=self.config)

    def get_screen(self):
        return LayoutScreen(
            element_type=LayoutWidget.tool,
            tool=ToolWidget.html_inject,
            load_element=self.load_element,
            region=self.region,
            height=self.height,
            width=self.width,
            button_label=self.button_label,
        )

    def __str__(self):
        return f"Html inject load element {self.load_element}"


class ActionBlockOption:
    def __init__(self, title, value, module: str = ""):
        self.title = title
        self.value = value
        self.module = module


class ActionBlock:
    def __init__(
        self,
        width: str = "",
        height: str = "",
        button_label: str = "",
        question: str = "",
        region: Region = Region.none,
    ):
        self.config = {"options": []}
        self.load_element = 0
        self.height = height
        self.width = width
        self.button_label = button_label
        self.region = region
        self.question = question
        self.type = ToolWidget.action_block

    def add_option(self, option: ActionBlockOption):
        self.config["options"].append(option.__dict__)

    def get_tool(self):
        return ToolScreen(tool_code=self.type, config=self.config)

    def get_screen(self):
        return LayoutScreen(
            element_type=LayoutWidget.tool,
            tool=ToolWidget.html_inject,
            load_element=self.load_element,
            region=self.region,
            height=self.height,
            width=self.width,
            button_label=self.button_label,
        )

    def __str__(self):
        return f"Action block element {self.load_element}"


class HtmlElement:
    def __init__(
        self,
        width: str = "",
        height: str = "",
        button_label: str = "",
        region: Region = Region.none,
    ):
        self.config = {"html": ""}
        self.load_element = 0
        self.height = height
        self.width = width
        self.button_label = button_label
        self.region = region
        self.type = ToolWidget.html_element

    def add_html(self, html_content: str):
        self.config["html"] = html_content

    def get_tool(self):
        return ToolScreen(tool_code=self.type, config=self.config)

    def get_screen(self):
        return LayoutScreen(
            element_type=LayoutWidget.tool,
            tool=ToolWidget.html_element,
            load_element=self.load_element,
            region=self.region,
            height=self.height,
            width=self.width,
            button_label=self.button_label,
        )

    def __str__(self):
        return f"Html element load element {self.load_element}"


class Split:
    def __init__(
        self,
        orientation: OrientationLayout,
        children: List,
        button_label: str = "",
        width: str = "",
        height: str = "",
        region: Region = Region.none,
    ):
        try:
            orientation = orientation.value
        except AttributeError:
            orientation = orientation

        if orientation == OrientationLayout.horizontal.value:
            for element in children:
                try:
                    el_region = element.region.value
                except AttributeError:
                    el_region = element.region
                if el_region != Region.center.value:
                    assert el_region in [
                        Region.top.value,
                        Region.bottom.value,
                    ], f"Region of {element} must be 'top' or 'bottom'."
                assert (
                    element.height != ""
                ), f"'height' of {element} must be defined in {element.__str__()}."

        elif orientation == OrientationLayout.vertical.value:
            for element in children:
                try:
                    el_region = element.region.value
                except AttributeError:
                    el_region = element.region
                if el_region != Region.center.value:
                    assert el_region in [
                        Region.right.value,
                        Region.left.value,
                    ], f"Region of {element} must be 'left' or 'right'."
                assert (
                    element.width != ""
                ), f"'width' of {element} must be defined in {element.__str__()}."
        else:
            raise Exception("Orientation value is not valid.")

        children_list = list()
        for child in children:
            if not isinstance(child, dict):
                children_list.append(process_element(child.get_screen()))
            else:
                children_list.append(child)

        self.children = children_list
        self.orientation = orientation
        self.type = LayoutWidget.split.value
        self.button_label = button_label
        self.height = height
        self.width = width
        self.region = region.value

    def __repr__(self):
        return process_element(self.get_screen())

    def get_screen(self):
        return self

    def get_tool(self):
        return self


class Tab:
    def __init__(
        self,
        children: List,
    ):
        for element in children:
            assert (
                element.button_label != ""
            ), f"'button_label' of {element} must be defined."
        self.type = LayoutWidget.tab.value
        children_list = list()
        for child in children:
            if not isinstance(child, dict):
                children_list.append(process_element(child.get_screen()))
            else:
                children_list.append(child)
        self.children = children_list

    def get_screen(self):
        return self

    def get_tool(self):
        return self

    def __repr__(self):
        return process_element(self.get_screen())


def process_element(new_element):
    return dict((k, v) for k, v in new_element.__dict__.items() if bool(v))


class ResultsConfiguration:
    def __init__(self):
        self.screen = dict()
        self.tools = list()
        self.load_element = 1
        self.elements = dict()

    def add_visualization(
        self,
        new_element: Union[PapayaViewer, HtmlElement, HtmlInject, ActionBlock],
    ):
        new_element.load_element = self.load_element
        tool_dict = process_element(new_element.get_tool())
        self.load_element += 1
        self.tools.append(tool_dict)

    def generate_results_configuration_file(
        self,
        build_screen: Union[List, Tab, Split],
        tool_path: str,
        testing_configuration: bool = False,
    ):
        if isinstance(build_screen, list):
            for new_element in build_screen:
                screen_dict = process_element(new_element.get_screen())
                self.screen.update(screen_dict)
        else:
            self.screen.update(process_element(build_screen.get_screen()))

        output = {"screen": self.screen, "tools": self.tools}
        encoded = json.dumps(output, indent=2)
        if testing_configuration:
            return output
        with open(
            os.path.join(tool_path, "results_configuration.json"), "w"
        ) as f:
            f.write(encoded)



#! /usr/bin/env python

import os
import re
import sys
from tkinter import CENTER, messagebox

import ttkbootstrap as ttk
from PIL import ImageTk, Image
from qmenta.sdk.tool_maker.make_files import (
    build_local_dockerfile,
    build_local_requirements,
    build_script,
    build_test,
    build_description,
    raise_if_false,
)

FONT = "Consolas"
FONTSIZE = 10

MIN = 0
MAX_CORES = 50
MAX_RAM = 150

FOLDER = "local_tools"
sys.path.append(FOLDER)


def gui_tkinter():
    def generate_output_object():
        values = {
            "tool_id": tool_id_entry.get(),
            "tool_version": tool_version_entry.get(),
        }
        return values

    def perform_selection_check(values):
        try:
            raise_if_false(
                isinstance(values["tool_id"], str), "Tool ID must be a string."
            )
            raise_if_false(values["tool_id"] != "", "Tool ID must be defined.")
            values["tool_id"] = values["tool_id"].lower()  # must be lowercase
            # start with letter, contain at least 3 letters, only a-z and _, cannot end with _
            raise_if_false(
                bool(re.match(r'^(?=(?:_*[a-z]){3})[a-z][a-z_]*[a-z]$', values["tool_id"])),
                "Tool ID can only contain letters and _",
            )

            values["folder"] = os.getcwd()
            if os.path.exists(
                os.path.join(values["folder"], values["tool_id"])
            ):
                messagebox.showerror(
                    "Python Error", "Error: This is an Error Message!"
                )
            return values
        except AssertionError as e:
            messagebox.showerror("Error", f"AN EXCEPTION OCCURRED! {e}")
            return False

    # window
    window = ttk.Window(themename="darkly")
    window.title("Tool Maker GUI")
    # image
    logo = Image.open(
        os.path.join(
            os.path.dirname(__file__), "templates_tool_maker", "qmenta.png"
        )
    ).resize((500, 110))
    img = ImageTk.PhotoImage(logo)
    image_label = ttk.Label(master=window, image=img, padding=5, anchor=CENTER)
    image_label.config(image=img)
    image_label.pack(side="top")

    # title
    title_label = ttk.Label(
        master=window,
        text="Fill the fields and click create to automatically generate a new tool file structure.",
        font="bold",
    )
    title_label.pack(padx=10, pady=10)

    # input tool ID field
    tool_id_frame = ttk.Frame(master=window)
    tool_id_label = ttk.Label(
        master=tool_id_frame, text="Specify the tool ID.*"
    )
    tool_id_label.pack(side="left")
    tool_id_entry = ttk.Entry(master=tool_id_frame)
    tool_id_entry.pack(side="left", padx=10)
    tool_id_frame.pack(pady=10)

    # input tool version field
    tool_version_frame = ttk.Frame(master=window)
    tool_version_label = ttk.Label(
        master=tool_version_frame, text="Specify the tool version.*"
    )
    tool_version_label.pack(side="left")
    tool_version_entry = ttk.Entry(master=tool_version_frame)
    tool_version_entry.insert(0, "1.0")
    tool_version_entry.pack(side="left", padx=10)
    tool_version_frame.pack(pady=10)

    gui_content = {}

    def submit_form():
        gui_content.update(generate_output_object())
        gui_content.update(perform_selection_check(gui_content))
        if gui_content:
            window.destroy()

    action_frame = ttk.Frame(master=window)
    button = ttk.Button(
        master=action_frame,
        text="Create",
        command=submit_form,
        bootstyle="primary",
    )
    button.pack(side="left")
    action_frame.pack(pady=10)

    def quit_tool_creator():
        window.destroy()
        print("GUI closed.")
        exit()

    ttk.Button(
        action_frame,
        text="Quit",
        command=quit_tool_creator,
        bootstyle="danger",
    ).pack(side="left", padx=10)

    # run
    window.mainloop()
    return gui_content


def main():
    content_build = gui_tkinter()
    if not content_build:
        exit()

    os.makedirs(os.path.join(FOLDER, content_build["tool_id"]), exist_ok=True)
    os.chdir(os.path.join(FOLDER, content_build["tool_id"]))
    build_description()
    build_script(code=content_build["tool_id"])
    with open("version", "w") as f1:
        f1.write(content_build["tool_version"])

    os.makedirs("local", exist_ok=True)
    os.chdir("local")

    build_local_requirements()
    build_local_dockerfile()
    build_test(
        content_build["tool_id"],
        FOLDER,
        content_build["tool_version"],
    )


if __name__ == "__main__":
    main()



import json
import logging
import os
import shutil
from subprocess import check_output, CalledProcessError, STDOUT

from qmenta.sdk.directory_utils import mkdirs, TemporaryDirectory


def tag_files(files):
    # TODO: improve tagging
    tfiles = []

    for f in files:
        tags = []

        # Modality
        if "T1w" in f:
            tags.append("m:T1")
        elif "T2w" in f:
            tags.append("m:T2")

        # Tags
        if "brainmask" in f:
            tags.append("mask")
        elif "labels" in f:
            tags.append("labels")
            if "tissue_labels" in f:
                tags.append("tissue_segmentation")
        elif "biasfield" in f:
            tags.append("bf")
        elif "restore" in f:
            tags.append("bfc")
        elif "anat2" in f or "2anat" in f:
            tags.append("warp")

        tfiles.append((f, tags))

    return tfiles


def generate_file_path(original_name, sub_name, modality_name, cat_dir):
    """
    Returns a valid BIDS name (full path) for the given file name.

    Parameters
    ----------
    original_name : str
        Original name of the file.
    sub_name : str
        QMENTA platform subject name (SUB-XXX_...).
    modality_name : str
        Modality string in the BIDS format (may be part of the name).
    cat_dir : str
        Path to the directory where the file will be stored.

    Returns
    -------
    str
        The full BIDS-compatible path where the file should be downloaded.
    """

    # Try to detect if the file name is already in BIDS format
    if original_name.startswith("sub-") and modality_name in original_name:

        # Use the original name (replacing the subject ID)
        target_name = "_".join([sub_name] + original_name.split("_")[1:])

    else:

        # Automatically generate a BIDS compatible name for the file
        extension = "." + ".".join(original_name.split(".")[1:])
        if extension in [
            ".bval",
            ".bvec",
        ]:  # In BIDS, bval and bvec use dwi as modality
            target_name = "{}_dwi{}".format(sub_name, extension)
        else:
            target_name = "{}_{}{}".format(sub_name, modality_name, extension)

    return os.path.join(cat_dir, target_name)


def format_input_data(context, subject, session, path):
    """
    Format input data followings the BIDS specification: http://bids.neuroimaging.io/bids_spec.pdf

    :param context: AnalysisContext of the running analysis
    :param subject: Subject name (from analysis_data)
    :param session: Session ID (from analysis_data)
    :param path: Destination directory where the files will be placed

    TODO: support group analyses (multiple input containers).
    """

    logger = logging.getLogger(__name__)

    dataset_description = {"Name": "QMENTA_ANALYSIS", "BIDSVersion": "1.1.0"}

    # The input container specification must include file filters following the bids_ff_names nomenclature
    bids_ff_names = {
        "anat": [
            "T1w",
            "T2w",
            "T1rho",
            "T1map",
            "T2map",
            "T2star",
            "FLAIR",
            "FLASH",
            "PD",
            "PDmap",
            "PDT2",
            "inplaneT1",
            "inplaneT2",
            "angio",
        ],
        "func": ["bold", "events", "physio", "sbref"],
        "dwi": ["dwi", "bval", "bvec"],
        "fmap": ["phasediff, magnitude1"],
    }

    supported_extensions = [".nii", ".nii.gz", ".bval", ".bvec", ".tsv"]

    # BIDS spec does not allow underscores or dashes
    sub_name = "sub-{}".format(subject.replace("-", "").replace("_", ""))
    sub_path = os.path.join(path, sub_name)
    mkdirs(sub_path)

    with open(os.path.join(path, "dataset_description.json"), "w") as fp:
        json.dump(dataset_description, fp)

    for category in list(bids_ff_names.keys()):
        for modality_name in bids_ff_names[category]:
            try:
                file_handlers = context.get_files(
                    "input",
                    file_filter_condition_name="c_{}".format(modality_name),
                )
            except:  # noqa: E731,E123,E722
                continue

            for fh in file_handlers:

                # NIFTI IS A MUST FOR BIDS
                if any(
                    [fh.name.endswith(ext) for ext in supported_extensions]
                ):
                    cat_dir = os.path.join(sub_path, category)
                    mkdirs(cat_dir)

                    target_path = generate_file_path(
                        fh.name, subject, modality_name, cat_dir
                    )

                    if os.path.isfile(target_path):
                        logger.warning(
                            "Multiple files with same name ({}). Only the first will be used".format(
                                target_path
                            )
                        )
                        break

                    with TemporaryDirectory() as tmp_dir:
                        download_path = fh.download(tmp_dir)
                        shutil.move(download_path, target_path)
                        logger.info("File: {}".format(target_path))

                    metadata = fh.get_file_info()
                    if metadata:
                        metadata_target_name = "{}_{}{}".format(
                            sub_name, modality_name, ".json"
                        )
                        metadata_target_path = os.path.join(
                            cat_dir, metadata_target_name
                        )
                        with open(metadata_target_path, "w") as fp:
                            json.dump(metadata, fp)
                else:
                    logger.warning(
                        "File {} extension not supported ({})".format(
                            fh.name, supported_extensions
                        )
                    )


def format_settings(settings):
    # Bypass from JSON schema
    # See docs.qmenta.com for more information

    # FIXME: Better define inputs, for now, we'll skip common settings
    ignore_param_list = ["input", "age_months"]

    options = []
    for param in settings:
        if param not in ignore_param_list:
            options += ["--" + param, str(settings[param])]
    return options


def run(context):
    logger = logging.getLogger(__name__)

    # Get information from the platform such as patient name, user_id, ssid...
    analysis_data = context.fetch_analysis_data()
    context.set_progress(message="Preparing data...")

    # Create in/out dirs
    input_path = "/analysis/input/"
    output_path = "/analysis/output/"
    mkdirs(input_path)
    mkdirs(output_path)

    # Download data from the platform and store following the BIDS specification
    format_input_data(
        context,
        subject=analysis_data["patient_secret_name"],
        session=analysis_data["ssid"],
        path=input_path,
    )

    # Select type of analysis (TODO: add support for group analysis)
    analysis_type = "participant"

    # Prepare a list of arguments from the settings
    options = format_settings(context.get_settings())

    # Run the original BIDS-APP entrypoint
    app_path = os.environ["BIDS_APP_ENTRYPOINT"]
    context.set_progress(
        message="Running {}".format(os.path.basename(app_path))
    )
    try:
        args = [app_path, input_path, output_path, analysis_type] + options
        logger.info("Calling: {}".format(" ".join(args)))
        ret = check_output(args, stderr=STDOUT)
        logger.info(ret)
    except CalledProcessError as e:
        raise RuntimeError(
            "Program exited with error: {}\nOutput:\n{}".format(e, e.output)
        )
    except Exception:
        raise

    # Upload output files
    context.set_progress(message="Saving output...")
    output_files = []
    for dirpath, _, filenames in os.walk(output_path):
        for f in filenames:
            file_path = os.path.abspath(os.path.join(dirpath, f))
            output_files.append(file_path)

    for f, tags in tag_files(output_files):
        output_container_path = os.path.relpath(f, output_path)
        context.upload_file(f, output_container_path, tags=tags)



import json

import logging

recognized_types = {
    "container",
    "string",
    "integer",
    "decimal",
    "single_choice",
    "multiple_choice",
}

non_supported_types = {"subject", "file"}


def parse_tool_settings(settings_path, settings_values_path):
    """
    Parse tool settings and their values

    Parameters
    ----------
    settings_path : str
        Path to the settings file.
        Uses JSON format and same syntax as platform
    settings_values_path : str
        Path to the settings' values file. Uses JSON format.

    Returns
    -------
    dict
        Dictionary with key-value settings
    """
    logger = logging.getLogger(__name__)

    settings_description = None
    with open(settings_path, "r") as fd:
        try:
            settings_description = json.load(fd)
        except ValueError:
            raise SettingsException(
                "{} does not have a proper JSON format".format(settings_path)
            )

    input_settings = None
    with open(settings_values_path, "r") as fd:
        try:
            input_settings = json.load(fd)
        except ValueError:
            raise InputSettingsException(
                "{} does not have a proper JSON format".format(
                    settings_values_path
                )
            )

    out_dict = dict()
    for elem in settings_description:
        setting_type = elem["type"]
        if setting_type in non_supported_types:
            logger.warning(
                "The setting type {} is not supported".format(setting_type)
            )
            continue
        elif setting_type in recognized_types:
            setting_id = elem["id"]
            # Assume mandatory is False if not set --> optional is True
            mandatory = elem.get("mandatory", False)
            out_dict[setting_id] = None
            if "default" in elem:
                out_dict[setting_id] = elem["default"]
            if setting_id in input_settings:
                out_dict[setting_id] = input_settings[setting_id]
            if mandatory and out_dict[setting_id] is None:
                raise InputSettingsException(
                    "Setting with id {} requires a value of type {}".format(
                        setting_id, setting_type
                    )
                )
    return out_dict


class SettingsException(Exception):
    pass


class InputSettingsException(Exception):
    pass



import logging
import os
import shutil
import zipfile

from enum import Enum

from qmenta.sdk.context import _should_include_file
from qmenta.sdk.directory_utils import TemporaryDirectory


class LocalAnalysisContext:
    class QCEntity(Enum):
        """
        Enum with the following options:
        ANALYSIS, SESSION
        """

        ANALYSIS = "analysis"
        SESSION = "patients"

    class QCStatus(Enum):
        """
        Enum with the following options:
        FAIL, PASS
        """

        PASS = "pass"
        FAIL = "fail"

    def __init__(self, settings, src_folder, out_folder, res_folder):
        """
        Context object that interfaces with the QMENTA platform. Should not be created directly.
        """

        self.__settings = settings
        self.__src_folder = src_folder
        self.__out_folder = out_folder
        self.__res_folder = res_folder

    def fetch_analysis_data(self):
        """
        Fetch information about current analysis.

        Returns
        -------
        dict
            Dictionary that contains important information about the current analysis, such as:
                * state (str) : State of the analysis
                * name (str) : Name of the analysis
                * script_name (str) : Name of the script
                * user_id (str) : Username of the user that started the analysis
                * patient_secret_name (str) : Name of the subject
                * ssid (str) : Timepoint identifier
                * settings (dict) : Settings dictionary. See documentation in method `get_settings`
                * tags (list) : List of tags
                * version (str) : Tool version
        """
        return {
            "state": "local_running",
            "name": "local",
            "script_name": "local",
            "user_id": "local",
            "patient_secret_name": "local",
            "ssid": "local",
            "settings": self.__settings,
            "tags": "local",
            "version": "local",
        }

    def set_qc_status(
        self,
        status=QCStatus.PASS,
        comments="",
        entity=QCEntity.ANALYSIS,
        input_id=None,
    ):
        """
        Changes the analysis or session QC status.

        Parameters
        ----------
        status : QCStatus
            QCStatus.PASS or QCStatus.FAIL
        comments : str, optional
            Additional comments explaining why the QC status has been set to pass or fail.
        entity : QCEntity, optional
            QCEntity.ANALYSIS or QCEntity.SESSION
        input_id : str, optional
            The id of the session in the settings definition when entity is QCEntity.SESSION
        """
        logger = logging.getLogger(__name__)

        logger.info("Setting QC status to {}: {}".format(status, comments))

    def get_qc_status(self, patient_secret_name=None, ssid=None):
        """
        Changes the analysis or session QC status.

        """
        logger = logging.getLogger(__name__)

        logger.info("Getting QC status")

    def download_session_qa_requirements(self, path):
        """
        Downloads the list of project-level requirements for a session.

        Parameters
        ----------
        path : str
            The location where to store the requirements.r
        """
        logger = logging.getLogger(__name__)

        if os.path.exists(path):
            raise RuntimeError(
                "Cannot download session qa because {} already exists!".format(
                    path
                )
            )

        logger.info(
            "Session QA requirements not supported locally. Skipping..."
        )

    def upload_session_qa_report(self, input_id, path):
        """
        Sets the QA status for all the files of session as per requirements

        Parameters
        ----------
        input_id : str
            Identifier "id" of the input container in advanced settings.
        path : str
            The json file with the QA status for all files and rules
        """
        logger = logging.getLogger(__name__)
        logger.info(
            "Session QA requirements not supported locally. Skipping..."
        )

    @staticmethod
    def set_progress(message=None, value=None):
        """
        Sets analysis progress.

        Parameters
        ----------
        value : int, optional
            Number between 0 and 100 indicating the current status of the execution
        message : str, optional
            Progress message.
        """
        logger = logging.getLogger(__name__)

        if not value and not message:
            logger.warning(
                "No information is available to report the progress"
            )
            return

        if value:
            value = int(value)  # Cast to int to avoid float/string problems
            if value > 100:
                logger.warning(
                    "The progress value should be between 0 and 100 (using 100 instead)"
                )
                value = 100
            elif value < 0:
                logger.warning(
                    "The progress value should be between 0 and 100 (using 0 instead)"
                )
                value = 0

        logger.info("Progress: {} -> {}".format(value, message))

    def get_settings(self):
        """
        Analysis settings.

        Returns
        -------
        dict
            Settings for the current analysis.
        """
        return self.__settings

    def get_files(
        self,
        input_id,
        modality=None,
        tags=None,
        reg_expression=None,
        file_filter_condition_name=None,
        subject_container_id=None,
    ):
        """
         Returns the files in the input container specified by `input_id` that match the modality, tags
         and regular expression, if any.

         Parameters
         ----------
        input_id : str
             Identifier "id" of the input container in advanced settings.
         modality : str, optional
             Optional modality, will allow any modalities if None given.
             Default is None.
         tags : set, optional
             Optional set of tags, will allow any tags if None given, and will match any set of tags (including
             the empty set) otherwise.
             Default is None.
         reg_expression : str, optional
             Regular expression to match with the file names in the specified container, will allow any file names
             if None given.
             Default is None.
         file_filter_condition_name : str, optional
             File filter specified in the settings of the tool.
             Default is None.

         Returns
         -------
         list of File
             List of selected file handlers from the container.
        """
        logger = logging.getLogger(__name__)
        logger.info(
            "Get files: {} [modality={} / tags={} / regex={}]".format(
                input_id, modality, tags, reg_expression
            )
        )

        file_handlers = []

        # Create File
        for file_description in self.__settings[input_id]:
            if "path" not in file_description:
                msg = 'Cannot find the key "path" for the input container {}'.format(
                    input_id
                )
                logger.error(msg)
                raise RuntimeError(msg)

            # Use file_filter_condition_name if available
            if file_filter_condition_name:
                if "file_filter_condition_name" not in file_description:
                    continue  # Skip
                else:
                    desc_ff_name = file_description[
                        "file_filter_condition_name"
                    ].replace("c_", "")
                    func_ff_name = file_filter_condition_name.replace("c_", "")
                    if func_ff_name != desc_ff_name:
                        continue  # Skip

            if "file_info" in file_description:
                assert isinstance(file_description["file_info"], dict)

            fmodality = file_description.get("modality", None)
            ftags = file_description.get("tags", set())
            finfo = file_description.get("file_info", dict())
            if _should_include_file(
                os.path.basename(file_description["path"]),
                fmodality,
                set(ftags),
                modality,
                tags,
                reg_expression,
            ):
                fh = LocalFile(
                    file_description["path"],
                    os.path.join(self.__src_folder, str(input_id)),
                    modality=fmodality,
                    tags=ftags,
                    file_info=finfo,
                )
                file_handlers.append(fh)

        return file_handlers

    def upload_file(
        self,
        source_file_path,
        destination_path,
        modality=None,
        tags=None,
        file_info=None,
        file_format=None,
        protocol=None,
        container_id=None,
        replace=None,
        direct=False,
    ):
        """
        Upload a file to the platform, to be part of the result files.

        Parameters
        ----------
        source_file_path : str
            Path to the file to be uploaded.
        destination_path : str
            Path in the output container. Will be shown in the platform.
        modality : str
        tags : None
            Only None is currently accepted for local testing
        file_info : None
        file_format : None
        protocol : None
        container_id : None
        """
        logger = logging.getLogger(__name__)
        if tags:
            logger.warning("Tags ignored (local execution): {}".format(tags))
        if modality:
            logger.warning(
                "Modality ignored (local execution): {}".format(modality)
            )
        if file_info:
            logger.warning(
                "File information ignored (local execution): {}".format(
                    file_info
                )
            )
        if file_format:
            logger.warning(
                "File information ignored (local execution): {}".format(
                    file_format
                )
            )
        if container_id:
            logger.warning(
                "File information ignored (local execution): {}".format(
                    container_id
                )
            )

        out_path = os.path.join(self.__out_folder, destination_path)
        os.makedirs(os.path.dirname(out_path), exist_ok=True)
        logger.info(
            "Uploading {!r} to {!r}".format(source_file_path, out_path)
        )
        shutil.copyfile(source_file_path, out_path)

    def download_resource(self, resource_path, destination_path):
        """
        Downloads a file from the user/group tool resources. The resource path can include subdirectories and must
        be relative:
        Downloads a resource file to the container.

        Parameters
        ----------
        resource_path : str
            Path to the file in the resources fi
        destination_path : str
            Path where the file will be downloaded to
        """
        logger = logging.getLogger(__name__)
        logger.info(self.__res_folder)
        if not self.__res_folder:  # Default keyword for external users
            logger.error("Resources are not properly configured. Ignoring...")
            return
        else:
            logger.info(
                "Mocking resources from directory: {}".format(
                    self.__res_folder
                )
            )
            logger.info(
                "The selected mode ({}) is ignored in local execution.".format(
                    type
                )
            )

        logger.info(
            "Downloading resource {!r} to {!r}".format(
                resource_path, destination_path
            )
        )

        os.makedirs(os.path.dirname(destination_path), exist_ok=True)
        try:
            shutil.copyfile(
                os.path.join(self.__res_folder, resource_path),
                destination_path,
            )
            logger.debug("Saving {!r}".format(destination_path))
        except IOError:
            logger.error("Cannot find the requested resource file.")

    def _get_manual_analysis_data(self):
        """
        Returns a dictionary with the manual analysis data generated during the manual step (user interaction)

        Returns
        -------
        dict
            The values generated during the manual step.
        """
        logger = logging.getLogger(__name__)
        logger.info("Getting manual analysis data")
        logger.warning(
            "Getting manual analysis data is not supported locally. Skipping..."
        )
        return {}

    def set_metadata_value(
        self,
        key,
        value,
        title=None,
        readonly=False,
        patient_secret_name=None,
        ssid=None,
    ):
        """
        Sets the value of a metadata parameter.

        Parameters
        ----------
        key : str
            The ID of the metadata parameter.
        value : int, ,str, float or list
            The new content of the parameter.
        title : str, optional
            How the metadata field should be presented
        readonly : bool, optional
            Whether the user should be able to edit the value on the platform or not (only by analyses)
        """
        logger = logging.getLogger(__name__)

        assert isinstance(key, str)
        assert isinstance(value, (int, list, str, float))
        if title:
            assert isinstance(title, str)

        allowed_types = {
            int: "integer",
            list: "list",
            str: "string",
            float: "decimal",
        }
        if type(value) not in allowed_types:
            raise ValueError(
                "Invalid type for the metadata value: {!r}".format(type(value))
            )

        logger.info("Setting metadata parameter: {} = {!r}".format(key, value))
        logger.warning(
            "Setting metadata values is not supported locally. Skipping..."
        )
        return None

    def get_metadata_value(self, key, patient_secret_name=None, ssid=None):
        """
        Gets the value of a metadata parameter for the current session

        Parameters
        ----------
        key : str
            The ID of the metadata parameter.
        patient_secret_name : str, optional
            The subject ID of the metadata parameter.
        ssid : str, optional
            The session ID of the metadata parameter.

        Returns
        -------
        The value of the parameter if it exists, None otherwise
        """
        logger = logging.getLogger(__name__)
        logger.info("Getting metadata value: {}".format(key))
        logger.warning(
            "Getting metadata values is not supported locally. Skipping..."
        )
        return None

    def _assure_metadata_parameters(self, params):
        """
        Check whether a metadata parameter exists, creating it if not. Not supported for local testing.

        Parameters
        ----------
        params : dict
            The definition of the parameter. It may contain "title", "type", "visible", "readonly" or "order".
        """
        logger = logging.getLogger(__name__)
        logger.info("Setting metadata param (ckeck): {}".format(params))
        logger.warning(
            "Assuring metadata params is not supported locally. Skipping..."
        )

    def set_analysis_output_variables(
        self, variables_dict, clear_previous=False
    ):
        """
        Sets the output variables of analysis

        Parameters
        ----------
        variables_dict : str
            Dictionary with the output variables
        clear_previous : bool, optional
            Whether the previous output variables should be cleared or not
        """
        logger = logging.getLogger(__name__)
        logger.warning("Output variables not supported locally. Skipping...")


class LocalFile:
    def __init__(self, name, input_container_path, modality, tags, file_info):
        """
        Object that represents a file and all its metadata in the platform. Should not be created directly.
        """
        self.name = name
        self.__input_container_path = input_container_path
        self.__download_path = None
        self.__modality = modality
        self.__tags = tags
        self.__file_info = file_info

    def get_file_modality(self):
        """
        Get the modality of the file.

        Returns
        -------
        str or None
            Modality of the file or None if not known.
        """
        return self.__modality

    @staticmethod
    def get_file_format():
        """
        Get the format of the file (e.g. 'nifti', 'txt', etc.).

        Returns
        -------
        str or None
            File format or None if not known.
        """
        return None

    def get_file_info(self):
        """
        Get the file information. The type of information depends on the type of file
        (e.g. nifti files include information such as 'Data strides', 'Data type', 'Dimensions' or 'Voxel size').

        Returns
        -------
        dict
            Dictionary of key-value pairs that depend on the format of the file.
        """
        return self.__file_info

    def get_file_tags(self):
        """
        Get the file tags.

        Returns
        -------
        set
            Set of tags associated to this file
        """
        return self.__tags

    def get_file_path(self):
        """
        Get the file download path.

        Returns
        -------
        str
            The path of the file where it has been downloaded

        Raises
        ------
        RuntimeError
            If `get_file_path` is called before the file has been downloaded
        """
        if self.__download_path is None:
            err_msg = "File {!r} has no path because it has not been downloaded".format(
                self.name
            )
            logging.getLogger(__name__).error(err_msg)
            raise RuntimeError(err_msg)
        else:
            return self.__download_path

    def __download_file(self, destination_file_path, direct=False):
        """
        Downloads the file into the specified `destination_file_path`.

        Parameters
        ----------
        destination_file_path : str
            Path where the file should be downloaded.
        """
        logger = logging.getLogger(__name__)
        logger.info(
            "Downloading {!r} to {!r}".format(self.name, destination_file_path)
        )
        os.makedirs(os.path.dirname(destination_file_path), exist_ok=True)
        source_file = os.path.join(self.__input_container_path, self.name)
        shutil.copyfile(source_file, destination_file_path)
        self.__download_path = destination_file_path

    def download(self, dest_path, unpack=True):
        """
        Downloads a file or the contents of a packed file to to the specified `path`.

        Parameters
        ----------
        dest_path:
            Path where the file should be downloaded to.
        unpack:
            Tells the function whether the file should be unpacked to the given folder.

        Returns
        -------
        str:
            The full path of the file or the folder with the unpacked files.
        """
        logger = logging.getLogger(__name__)
        source_file = self.name

        # Normalize path
        dest_path = os.path.abspath(dest_path)

        logger.debug("Using path {!r}".format(dest_path))

        if source_file.endswith(".zip") and unpack:
            # Download the zip to a temporary directory and unpack its contents to the user dir path
            with TemporaryDirectory() as temp_dir:
                temp_path = os.path.join(temp_dir, source_file)
                self.__download_file(temp_path)
                with zipfile.ZipFile(temp_path, "r") as zip_ref:
                    file_list = zip_ref.namelist()
                    logger.info(
                        "Decompressing {!r} to {!r}".format(
                            file_list, dest_path
                        )
                    )
                    os.makedirs(dest_path, exist_ok=True)
                    zip_ref.extractall(path=dest_path)

            self.__download_path = (
                dest_path  # Replace download path with directory
            )
            return dest_path
        else:
            destination_file_path = os.path.join(dest_path, source_file)
            self.__download_file(destination_file_path)
            return self.__download_path



import logging
from contextlib import contextmanager


class LogCaptureHandler(logging.Handler):
    """
    Log handler that captures all messages that it handles and exposes them as a list in `log_sequence` attribute.
    """

    def __init__(self):
        self.log_sequence = list()
        super(LogCaptureHandler, self).__init__()

    def emit(self, record):
        self.log_sequence.append(record)


class GlobFilter(logging.Filter):
    def __init__(self, glob_expression):
        self.__glob_sequence = glob_expression.split(".")
        super(GlobFilter, self).__init__()

    def filter(self, record):
        expected_seq = record.name.split(".")
        if len(expected_seq) == len(self.__glob_sequence):
            for expected, actual in zip(expected_seq, self.__glob_sequence):
                if actual != "*" and actual != expected:
                    return False
            return True
        else:
            return False


@contextmanager
def capture_log(logger_name_glob, log_level=logging.INFO):
    """
    Context manager that captures the logs of all loggers that satisfy `logger_name_glob`
    with associated severity greater or equal than `log_level`.

    Parameters
    ----------
    logger_name_glob : str
        Glob expression that specified the loggers should be captured.
    log_level : int, optional
        Minimum severity level to be captured, included. Default is logging.INFO.

    Returns
    -------
    LogCaptureHandler
        Handler that exposes the `log_sequence` attribute, which consists of a list of captured messages.

    Examples
    --------
    >>> logger = logging.getLogger('package.module.method')
    >>> with capture_log('package.module.*', log_level=logging.WARNING) as captured_logs:
    ...     logger.info("This info message won't be captured")
    ...     logger.warning("This warning message will be captured")
    ...     logger.critical("This critical message will be also captured")
    ...     print([record.msg for record in captured_logs.log_sequence])
    ['This warning message will be captured', 'This critical message will be also captured']
    """
    # Set up the handler, filter and log level
    root_logger = logging.getLogger()

    handler = LogCaptureHandler()
    handler.addFilter(GlobFilter(logger_name_glob))
    root_logger.addHandler(handler)

    default_level = root_logger.level
    root_logger.setLevel(log_level)

    yield handler

    # Clean up the handler and restore original log level
    root_logger.setLevel(default_level)
    root_logger.removeHandler(handler)
